class Wikian
  class WikianGetError < StandardError; end
  class ExtractWikiError < WikianGetError; end
  class ArgumentRequiredError < WikianGetError; end

  class Contributions < Subcommand
    DEFAULT_MAX_CONTRIBUTIONS = 20

    def initialize(args)
      super

      max_contributions = args.find(&:numeric?) || DEFAULT_MAX_CONTRIBUTIONS

      raise(BadUrlError, "Try passing the '-t' option") unless config['meta']['site']

      @output_file = 'User:' +  ENV['WIKI_USER'] + '.contributions.' + config['meta']['site']

      @params.merge!('ucuser' => ENV['WIKI_USER'], 'uclimit' => max_contributions, 'format' => Wikian::RESPONSE_FORMAT)

      @query = @params.to_query

      @api_url = URI("https://#{config['meta']['site']}/w/api.php?#{query}")
    rescue => e
      puts "#{e.class} in #{__FILE__}", e.message
      exit
    end

    def template
      <<~eos
        # for a list of parameters to use here see: https://www.mediawiki.org/wiki/API:Usercontribs
        meta:
          site: en.wikipedia.org
          headers:
            user-agent: Wikian
        api:
          action:
            - query
          list:
            - usercontribs
          ucprop:
            - title
            - comment
            - timestamp
            - sizediff
      eos
    end
  end
end
