class Wikian
  class WikianGetError < StandardError; end
  class ArgumentRequiredError < WikianGetError; end

  class Get < Subcommand
    attr_accessor :title, :url, :latest_revision

    def initialize(args)
      raise ArgumentRequiredError if args.empty?

      super

      @url = URI(args.find{|arg| arg =~ URI.regexp})

      raise BadUrlError unless url.path

      @title = File.basename(url.path)

      @output_file = title + '.' + url.host

      @params.merge!('titles' => title, 'format' => Wikian::RESPONSE_FORMAT)

      @query = @params.to_query

      @api_url = URI("https://#{url.host}/w/api.php?#{query}")
    rescue => e
      puts "#{e.class} in #{__FILE__}", e.message
      exit
    end

    # extract wikitext from the response file and save it into a `.wiki` file
    #
    # return: nil
    def extract_wikitext
      pages = JSON.parse(res_body).dig('query','pages')

      # Wikipedia is inconsistent in their value for 'pages', it's sometimes a hash, sometimes an array
      @latest_revision = (pages.respond_to?(:keys) ? pages.values.first : pages.first)['revisions'].first

      content = latest_revision.dig('slots', 'main', 'content') ||
                latest_revision.dig('slots', '*') ||
                latest_revision.dig('*')

      wiki_file= File.basename(response_file, File.extname(response_file)) + '.wiki'

      File.open(wiki_file,'w') do |f|
        STDERR.puts "Warning: nil 'content' in #{Wikian::CONFIG_FILE}" unless content
        STDERR.puts "Writing to #{wiki_file}"
        f.puts content
      end

    rescue => e
      puts "An error occurred while extracting the wikitext"
      exit
    end

    # save article metadata
    #
    # metadata like article timestamp is used to solve edit conflicts
    def save_metadata
      FileUtils.mkdir_p(Wikian.meta_dir)

      metadata = File.exist?(Wikian.meta_file) ? YAML.load(File.open(Wikian.meta_file)) : {'meta' => {}}
      metadata['meta'].merge!(title => {'timestamp' => latest_revision['timestamp']})

      File.write(Wikian.meta_file, YAML.dump(metadata))
    end

    def template
      <<~eos
        # for a list of parameters to use here see: https://www.mediawiki.org/wiki/API:Revisions
        meta:
          headers:
            user-agent: Wikian
        api:
          action:
            - query
          prop:
            - revisions
          rvprop:
            - content
            - timestamp
         #rvsection: # get specific sections
         #  - 0
         #  - 2
          rvslots:
            - main
          formatversion:
            - 2
      eos
    end
  end
end
